############################################################
## This LONG lab includes...
##
## A Cross-validation study for picking a model
## that appears to predict well
##
## The goal is to predict the remainder of 2016
## with a justified model
## 
## In doing so, we consider an assortment of candidate models
## including:
##  - Holt Winters
##  - Deterministic Models (Linear and Quadratic)
##  - ARIMA(p,d,q)x(P,D,Q) models
##
## From a Cross-validation study we choose a 
##    ARIMA(1,1,1)x(1,1,1)

library(forecast)
library(tseries)

tmp <- read.csv("natGasConsumption.csv")
nat.gas <- ts(tmp$U.S..Natural.Gas.Total.Consumption..MMcf., start=c(2001,1), frequency=12)

plot(nat.gas, main="U.S. Natural Gas Consumption", ylab="MMcf", type='o', pch=20)
plot(nat.gas, main="U.S. Natural Gas Consumption", ylab="MMcf")

### Our goal is to forecast the remainder of 2016... Our last observation
## is Jan 2016, so we want to forcast Feb 2016, Mar 2016.... through Dec 2016

nat.gas.train <- window((nat.gas), end=c(2015,1))
nat.gas.test <- window((nat.gas), start=c(2015,2), end=c(2015,12))

ts.plot(nat.gas.train,nat.gas.test, col=c("black", "blue"), lwd=c(1,2), lty=c(1,2))

###########################
## First let's fit a Holt Winters
hw.fit <- HoltWinters(nat.gas.train)

############################
## Now let's fit a big explanatory model
TIME <- 1:length(nat.gas.train)
SIN <- COS <- matrix(nrow=length(nat.gas.train), ncol=6)
for(i in 1:6) {
  COS[,i] <- cos(2*pi*i*TIME/12)
  SIN[,i] <- sin(2*pi*i*TIME/12)
}
SIN <- SIN[,-6]
lm.harmonic <- lm(nat.gas.train ~ TIME + COS + SIN )
summary(lm.harmonic)
####################
## Clearly we can remove some COS and SIN terms
## But first let's check the fit.
plot(as.ts(lm.harmonic$residuals))
###############
## See curvature?  Perhaps we need a quadratic?
lm.quad <- lm(nat.gas.train ~ TIME + I(TIME^2) + COS[,1:3] + SIN[,c(1,2,4,5)])
summary(lm.quad)
layout(1:3)
plot(lm.quad$residuals, type='l')
acf(lm.quad$residuals, lwd=2)
pacf(lm.quad$residuals, lwd=2)
layout(1:1)

##################
## Residuals look like an AR(1)
res.ar1 <- arima(lm.quad$residuals, order=c(1,0,0), seasonal=list(order=c(0,0,0), period=12), include.mean=FALSE)
res.ar1

layout(1:2)
acf(res.ar1$residuals, lwd=2)
pacf(res.ar1$residuals, lwd=2)
layout(1:1)

#################################
## Now refit the whole thing...
XREG <- cbind(TIME, I(TIME^2), COS[,1:3], SIN[,c(1,2,4,5)])
###########
## Name the columns of my XREG matrix for output readability
colnames(XREG) <- c("Time", "Time.sqr", "Cos.1", "Cos.2", "Cos.3", "Sin.1", "Sin.2", "Sin.4", "Sin.5")
fit.quad.det <- Arima(nat.gas.train, order=c(1,0,0), xreg=XREG)
plot(fit.quad.det$residuals)
acf(fit.quad.det$residuals, lwd=2)
pacf(fit.quad.det$residuals, lwd=2)
fit.quad.det
####################
## Although in a regression setting, the quadratic model might seem appropriate
## and this can even help explain the past... Over the past 15 years 
## Natural gas consumption has grown quadratically!... It seems
## squirrelly to use for forecasting.  Let's also include just a linear model
##
## Afterall, we are forecasting... What predicts the best is what matters
XREG <- cbind(TIME, COS[,1:3], SIN[,c(1,2,4,5)])
colnames(XREG) <- c("Time", "Cos.1", "Cos.2", "Cos.3", "Sin.1", "Sin.2", "Sin.4", "Sin.5")
fit.lin.det <- Arima(nat.gas.train, order=c(1,0,0), xreg=XREG)
fit.lin.det
########################
## Big models... note that all terms are significant
## when using the +/- 2 standard error criteria.
## 
## In fact, nearly every terms is well beyond two standard
## errors.  So everything looks significant.
##
## In an explanatory sense this tells me that the underlying trend
## in the Natural Gas series has increased (perhaps quadratically)... 
## Following something along the lines of 
##   Y = 1908463 - 2621*t + 29.2635*t^2 + seasonal_stuff + AR(1)_Errors
##
## But this lab is about forecasting... So let's see how it does for it.
##
## Now let's try an ARIMA approach.
## Original series is clearly not stationary
plot(nat.gas.train)
#########
## Differences..
plot(diff(nat.gas.train))
#######
## TANGENT... The differenced series clearly has seasonality (not-stationary)
## But what does Dickey-Fuller & Phillips-Perron tell us
adf.test(diff(nat.gas.train))
pp.test(diff(nat.gas.train))
#### Both say Stationary... when it is not; they have their limitations!
#########
## So clearly seasonal
## Seasonal differences?
plot(diff(nat.gas.train, lag=12))
#################
## Maybe this is okay? One could argue it follows a random walk?
## You could also argue it is stationary...
adf.test(diff(nat.gas.train, lag=12))
pp.test(diff(nat.gas.train, lag=12))
#########################
## They both say it is okay. My eyeball method says not sure
## Let's see what differencing a second time does does
plot(diff(diff(nat.gas.train,lag=12)))
###############
## Clearly stationary but correlated
acf(diff(diff(nat.gas.train,lag=12)), lwd=2)
pacf(diff(diff(nat.gas.train,lag=12)), lwd=2)

#########################
## From the above, this looks like it could be some sort of
## ARMA(p,q) with a SARMA(P,Q)...  
## Maybe like an ARMA(1,1)x(1,1) or something similar on the twice differenced series

###################
## What about the correlation structure in just the seasonal differenced series
acf(diff(nat.gas.train,lag=12), lwd=2)
pacf(diff(nat.gas.train,lag=12), lwd=2)

######################
## From these plots, this is a bit more clear
## It looks like an AR(1) plus another SAR(1) and/or SMA(1) term

### Let's check for drift terms too...
mean(diff(nat.gas.train,lag=12))   # Could be important
mean(diff(diff(nat.gas.train,lag=12)))  
# big number for seasonal differenced 
# Reasonably big for twice-differenced but consider the mean values of my data...
# Also, in general when you take multiple differences, you do not add drift terms

############################
############################
## At this point, if considering ARIMA type models
##
## 1. We need to take lag 12 differences (i.e. a seasonal difference)
## 2. We may or may not need to difference again, we will consider both
## 3. Both have some remaining serial correlation we will need to model
## 
## Let's try some different models.
## 
## First, the series that is differenced twice (once a lag 1 and another at lag 12)
Arima(nat.gas.train, order=c(1,1,0), seasonal=list(order=c(1,1,0), period=12)) # AICc=4105.87  BIC=4114.86
Arima(nat.gas.train, order=c(1,1,0), seasonal=list(order=c(0,1,1), period=12)) # AICc=4074.00  BIC=4082.99
Arima(nat.gas.train, order=c(1,1,0), seasonal=list(order=c(1,1,1), period=12)) # AICc=4074.97  BIC=4086.88
Arima(nat.gas.train, order=c(0,1,1), seasonal=list(order=c(1,1,0), period=12)) # AICc=4095.13  BIC=4104.12
Arima(nat.gas.train, order=c(0,1,1), seasonal=list(order=c(0,1,1), period=12)) # AICc=4065.33  BIC=4074.32
Arima(nat.gas.train, order=c(0,1,1), seasonal=list(order=c(1,1,1), period=12)) # AICc=4065.96  BIC=4077.89
Arima(nat.gas.train, order=c(1,1,1), seasonal=list(order=c(1,1,0), period=12)) # AICc=4076.68  BIC=4088.61
Arima(nat.gas.train, order=c(1,1,1), seasonal=list(order=c(0,1,1), period=12)) # AICc=4049.82  BIC=4061.76
Arima(nat.gas.train, order=c(1,1,1), seasonal=list(order=c(1,1,1), period=12)) # AICc=4050.45  BIC=4065.3

fit.arima111x011 <- Arima(nat.gas.train, order=c(1,1,1), seasonal=list(order=c(0,1,1), period=12))
fit.arima111x111 <- Arima(nat.gas.train, order=c(1,1,1), seasonal=list(order=c(1,1,1), period=12))
fit.arima111x011
fit.arima111x111
plot(fit.arima111x011$residuals)
acf(fit.arima111x011$residuals, lwd=2)
pacf(fit.arima111x011$residuals, lwd=2)

acf(fit.arima111x111$residuals, lwd=2)
pacf(fit.arima111x111$residuals, lwd=2)
##############################
## The two models seem almost identical... Similar AICc and BIC
## and residual plots look the same. In fact, the ARIMA(1,1,1)x(1,1,1)
## model include an insignificant term, so perhaps the ARIMA(1,1,1)x(0,1,1) 
## is the way to go?  We will consider both as viable forecasting options

#####################################################
## Okay... two candidate models when two differences
## Now let's consider just the seasonal differenced.
acf(diff(nat.gas.train, lag=12), lwd=2)
pacf(diff(nat.gas.train, lag=12), lwd=2)

Arima(nat.gas.train, order=c(1,0,0), seasonal=list(order=c(1,1,0),period=12)) # AICc=4100.42  BIC=4109.44
Arima(nat.gas.train, order=c(1,0,0), seasonal=list(order=c(0,1,1),period=12)) # AICc=4085.71  BIC=4094.72
Arima(nat.gas.train, order=c(1,0,0), seasonal=list(order=c(1,1,1),period=12)) # AICc=4087.01  BIC=4098.97
Arima(nat.gas.train, order=c(1,0,0), seasonal=list(order=c(1,1,0),period=12), include.drift=TRUE) # AICc=4098.62  BIC=4110.58
Arima(nat.gas.train, order=c(1,0,0), seasonal=list(order=c(0,1,1),period=12), include.drift=TRUE) # AICc=4077.40  BIC=4089.36
Arima(nat.gas.train, order=c(1,0,0), seasonal=list(order=c(1,1,1),period=12), include.drift=TRUE) # AICc=4076.00  BIC=4090.89
### We should include a drift term... or so AICc and BIC suggest
############
## For the ARIMA(1,0,0)x(0,1,1) and ARIMA(1,0,0)x(1,1,1) the AICs are within 2 and the BICs are about the same
## I will pick the simpler
fit.arima100x011 <- Arima(nat.gas.train, order=c(1,0,0), seasonal=list(order=c(0,1,1),period=12), include.drift=TRUE)
fit.arima100x011
acf(fit.arima100x011$residuals, lwd=2)
pacf(fit.arima100x011$residuals, lwd=2)
################# Perhaps missing something?  Add another SMA-term...
fit.arima100x012 <- Arima(nat.gas.train, order=c(1,0,0), seasonal=list(order=c(0,1,2),period=12), include.drift=TRUE)
fit.arima100x012  ## AIC gone down a bit, BIC about the same
acf(fit.arima100x012$residuals, lwd=2)
pacf(fit.arima100x012$residuals, lwd=2)
# A bit better?  
##
## We could go on forever trying more and more models, I am going to stop here...
## 
## I am going to settle for checking out the 
## ARIMA(1,0,0)x(0,1,1) and ARIMA(1,0,0)x(0,1,2), both with drift.

#############################################
## So my candidate predictive/forecasting Models...
##
## Holt-Winters
## Big Regression... 
## ARIMA(1,1,1)x(0,1,1)
## ARIMA(1,1,1)x(1,1,1)
## ARIMA(1,0,0)x(0,1,1) with drift
## ARIMA(1,0,0)x(0,1,2) with drift
## 
## Let's see what predicts the best against my testing set...

hw.forecast <- forecast(hw.fit, h=11)

############
## for the regression model... I need to do some programming tricks...

NEW.TIME <- seq(length(TIME)+1, length(TIME)+11, 1)   ### Continue my time values 11 more months
NEW.TIME.sqr <- NEW.TIME^2
NEW.SIN <- NEW.COS <- matrix(nrow=11, ncol=6, 0)
for(i in 1:6) {
  NEW.SIN[,i] <- sin(2*pi*i*NEW.TIME/12)
  NEW.COS[,i] <- cos(2*pi*i*NEW.TIME/12)
}
NEW.COS <- NEW.COS[,(1:3)]
NEW.SIN <- NEW.SIN[,c(1,2,4,5)]
NEW.XREG <- cbind(NEW.TIME, NEW.TIME.sqr, NEW.COS, NEW.SIN)
quad.forecast <- forecast(fit.quad.det, xreg=NEW.XREG)

NEW.XREG <- cbind(NEW.TIME, NEW.COS, NEW.SIN)
lin.forecast <- forecast(fit.lin.det, xreg=NEW.XREG)

##########
## Now my ARIMA forecast
forecast111x011 <- forecast(fit.arima111x011, h=11)
forecast111x111 <- forecast(fit.arima111x111, h=11)
forecast100x011 <- forecast(fit.arima100x011, h=11)
forecast100x012 <- forecast(fit.arima100x012, h=11)

plot(hw.forecast, include=25)
lines(nat.gas.test, lwd=2, lty=2)

plot(quad.forecast, include=25)
lines(nat.gas.test, lwd=2, lty=2)

plot(lin.forecast, include=25)
lines(nat.gas.test, lwd=2, lty=2)

plot(forecast111x011, include=25)
lines(nat.gas.test, lwd=2, lty=2)

plot(forecast111x111, include=25)
lines(nat.gas.test, lwd=2, lty=2)

plot(forecast100x011, include=25)
lines(nat.gas.test, lwd=2, lty=2)

plot(forecast100x012, include=25)
lines(nat.gas.test, lwd=2, lty=2)

fore.tab <- cbind(nat.gas.test, hw.forecast$mean, quad.forecast$mean, lin.forecast$mean,
                  forecast111x011$mean, forecast111x111$mean, 
                  forecast100x011$mean, forecast100x012$mean, nat.gas.test)
colnames(fore.tab) <- c("Observed", "Holt Winters", "Quad Model", "Lin. Model", "ARIMA(1,1,1)x(0,1,1)",
                        "ARIMA(1,1,1)x(1,1,1)", "ARIMA(1,0,0)x(0,1,1)", "ARIMA(1,0,0)x(0,1,2)", "Observed")
fore.tab

mspe.tab <- rbind( mean( (nat.gas.test - hw.forecast$mean)^2),
                   mean( (nat.gas.test - quad.forecast$mean)^2),
                   mean( (nat.gas.test - lin.forecast$mean)^2),
                   mean( (nat.gas.test - forecast111x011$mean)^2),
                   mean( (nat.gas.test - forecast111x111$mean)^2),
                   mean( (nat.gas.test - forecast100x011$mean)^2),
                   mean( (nat.gas.test - forecast100x012$mean)^2) )
colnames(mspe.tab) <- c("MSPE")
rownames(mspe.tab) <- c("Holt Winters", "Quad Model", "Lin. Model", "ARIMA(1,1,1)x(0,1,1)",
                        "ARIMA(1,1,1)x(1,1,1)", "ARIMA(1,0,0)x(0,1,1)", "ARIMA(1,0,0)x(0,1,2)")
mspe.tab
sqrt(mspe.tab)
###
# Looks like the ARIMA(1,1,1)x(1,1,1) is the best of these models by 
# a considerable amount, more than 1 billion squared MMcF or 7000 MMcf

#####
## Okay... now refit an ARIMA(1,1,1)x(1,1,1) to the whole data
## and use it to forecast the remainder of 2016
full.arima111x111 <- Arima(nat.gas, order=c(1,1,1), seasonal=list(order=c(1,1,1),period=12))
full.forecast <- forecast(full.arima111x111, h=11)
plot(full.forecast, include=25)
full.forecast

#############
## How well does this model do???  Check back next January when 2016 is over...
