require(tseries)
library(forecast)
require(vars)
############
## Load our Series
unemp.raw <- read.csv("unemployment.csv", skip=25)
gdp.raw <- read.csv("gdpData.csv", skip=19)
head(unemp.raw)
head(gdp.raw)

#############################
## Construct our Time Series objects
## 
## Take the log of the GDP as the economy grows exponentially in time
gdp <- ts(log(gdp.raw$VALUE), start=c(1980,1), frequency=4)
unemp.full <- ts(unemp.raw$VALUE, start=c(1980,1), frequency=4)
length(gdp)
length(unemp.full)

unemp <- window(unemp.full, end=c(2015,4))  ## Remove it

### This creates a bivariate time series object
finance <- ts.union(gdp, unemp)
plot(finance, type="l", lwd=2, col="gray10", main="Financial Series")

#########
## A basic idea from a long time ago
## decompose each series into the seasonal, trend and random part
## look at the cross-correlation between the random parts
## 
## Some indication GDP influences Unemployment...
gdp.dec <- stl(gdp, s.window="periodic")
unemp.dec <- stl(unemp, s.window="periodic")
ccf(gdp.dec$time.series[,3], unemp.dec$time.series[,3],
    main="CCF GDP & Unemployment", lwd=2)

################
## Let's start modeling our series marginally
##
## First we note both are not-stationary
## We look at first differences (to handle the stochastic trends)
## and lag-4 differences to handle the seasonality.
adf.test(gdp)
adf.test(unemp)
## Both are clearly not-stationary
## Perhaps they are Co-integrated??
help(po.test)
po.test(cbind(gdp, unemp))  
##
## We fail to reject the null hypothesis, so NO they are NOT cointegrated

#############
## What to do...
## We have quarlerly data, perhaps difference by quarter
plot(gdp)
plot(diff(gdp, lag=4))

plot(unemp)
plot(diff(unemp, lag=4))

diff.finance <- diff(finance, lag=4)

plot(diff.finance, type="l", lwd=3, col="gray10",
     main="Seasonally and Trend Adjusted Series (by differencing)")

############
## The data looks reasonably stationary now
##
## Check for serial correlation
## 
## We can also see a trend, when one goes up, the other goes down
##
## Math note... we are essentially modeling a derivative here, 
## since we are looking at differences (even if lagged by 4).
## When GDP changes up, it looks like unemployment changes with a decrease

acf(diff.finance, lwd=2)

## Look at the cross-correlation more-closely
ccf(diff.finance[,1], diff.finance[,2],
    main="CCF GDP & Unemployment", ylab="CCF")

#############
## We see some serial correlation in the marignal series along with 
## interaction in the cross-correlation.
##
## This tells me I might want to fit a vector AR model
##
## It turns out, if we want to explore the cross-correlation
## between two series, we also need to explore the marginal
## Serial correlations in the two series
## this is because we may want to fit an adequate VAR(p) model
## it is still there.
acf(diff.finance[,1], lwd=2)
pacf(diff.finance[,1], lwd=2)
###
## This looks like an AR(2) to me, maybe AR(5)?
acf(diff.finance[,2], lwd=2)
pacf(diff.finance[,2], lwd=2)  
###
## Also looks like an AR(2) or maybe AR(5)s?
## Perhaps a seasonal model?

#############
# let's try some Vector Autoregressive Models
VARselect(diff.finance)

##############
## Not entirely clear, some say AR(10). This seems pretty big
## given the data is quarterly? But one could argue it 
## also needs that far back because they are both measures of the
## economy which changes slowly?
my.var <- VAR(diff.finance,p=10)
my.var
summary(my.var)

###########################
## seems like overkill to me... Many non-significant terms
## 
## Let's try with p=2.  This is close to the 
## to the BIC (SC) criterion and the data is quarterly
## so it might capture some seasonal effects
my.var2 <- VAR(diff.finance, p=2)
my.var2
summary(my.var2)

### Which model is better? A matter of opinion I suppose
## 
## But with both models we can see something interesting
## Using the Granger test for causality
#######################
## Using the VAR(p=10) model
causality(my.var, cause="unemp")
causality(my.var, cause="gdp")

## Using the VAR(p=2) model
causality(my.var2, cause="unemp")
causality(my.var2, cause="gdp")

## The VAR(2) indicates the "causal" structure may be in both directions
## while the VAR(10) is pretty firm that GDP influences unemployment
##############

## We also  see the two series are instantaneously connected,
## This makes sense. Major spikes to the economy will have impact
## on both unemployment and GDP
##
## Overall we also see that GDP appears to "cause" unemployment.
## Furthermore, most of this causal effect is off by one-lag
## if you recall from the earlier CCF plot.
##
## So, a decrease in GDP this quarter, will cause unemployment to increase
## next quarter (also unemployment should decrease this quarter as well)
##
## Likewise, increase in GDP should cause unemployment to decrease
## In some of the economics literature this is known a Okun's Law

#######
## The vars packages has a neat feature
## in that it will plot your fitted model with the residuals and ACF/PACF
plot(my.var2)

###############
## Of course, we should look at the ACF of our 
## residuals to make sure we captured everything
## 
## For the interested reader, there are multivariate
## analogoues of the Ljung Box goodness-of-fit test, 
## see Hosking (1980)

## First the VAR(10)
acf(residuals(my.var))  # Nearly all serial correlation
acf(residuals(my.var2)) # not everything, something at lag 4!! maybe we should use the bigger model

###############
## and we can use the fitted model to forecast
my.var.predict <- forecast(my.var, h=10)
my.var.predict2 <- forecast(my.var2, h=10)
plot(my.var.predict, lwd=3)
plot(my.var.predict2, lwd=3)

### How does all this compare
diff.unemp.full <- diff(unemp.full, lag=4)
length(diff.unemp.full)
diff.unemp.full[141]

forecast(my.var, h=1)
forecast(my.var2, h=1)


