##################################################
###
### Analyzing Consumer Price Index (CPI) for the 
### past (roughly) 100-years
library(fGarch)
library(forecast)
library(tseries)
library(WeightedPortTest)

tmp <- read.csv("cpi.csv")
cpi <- ts(tmp$CPIAUCSL, start=c(1947,1), freq=12)
plot(cpi, main="Consumer Price Index of all Urban Consumers",
     sub="1982-1984=100 (Seasonally Adjusted)", ylab="CPI")

##############################
## For our analysis, we will only explore the CPI for the past 30-ish years
cpi <- window(cpi, start=c(1982,1))
plot(cpi, main="CPI of all Urban Consumers since 1982", 
     ylab="CPI", sub="Seasonally Adjusted")

###############
## Looks fairly linear
## but clearly has a trend, we take differences, divide by the previous cpi value and mulitple by 100
## this calculates inflation (seasonally adjusted)
inflation <- diff(cpi)/cpi[-399]*100
round(inflation, 2)
mean(inflation)     ### Inflation per month
                    ### Corresponds to about 2.8% per year, on average
#############
### Essentially we now have a measure of inflation per year.
plot(inflation, main="Inflation via CPI")
##############
### We see below that the inflation series is clearly serial-correlation
acf(inflation, lwd=2)
pacf(inflation, lwd=2)

###################################
### Feel free to try other fitted models, I settled on an ARIMA(0,0,2)(0,0,1)[12]
### on the inflation series. The purpose of this example is to model
### the conditional variance with a GARCH process.
fit <- Arima(inflation, order=c(0,0,2), seasonal=list(order=c(0,0,1),period=12))
fit
acf(fit$residuals, lwd=2)
pacf(fit$residual, lwd=2)
Weighted.Box.test(fit$residuals, lag=30, fitdf=3, type="Ljung")
Weighted.Box.test(fit$residuals, lag=30, fitdf=3, type="Monti")
#############################
## It is not perfect, but we seem to have an adequate model, good enough for now

###############################
## But if we explore the squared series, there is a changing variance
acf(fit$residuals^2, lwd=2)
pacf(fit$residuals^2, lwd=2)
#########################
### We can also statistically test this, using the McLeod Li test.
## This is essentially the Ljung-Box stuff but on the squared residuals.
Weighted.Box.test(fit$residuals, lag=30, type="Ljung", sqrd.res=TRUE)
Weighted.Box.test(fit$residuals, lag=30, type="Ljung", sqrd.res=TRUE, weighted=FALSE)
Weighted.Box.test(fit$residuals, lag=30, type="Monti", sqrd.res=TRUE)
Weighted.Box.test(fit$residuals, lag=30, type="Monti", sqrd.res=TRUE, weighted=FALSE)
###########################
### Clearly have ARCH/GARCH-type effects in our residuals, we should model it
### Here are two ways to model it, fitting a GARCH(1,1)

fit.g <- garch(fit$residuals, order=c(1,1))   # order=c(q,p) q- GARCH part, p - ARCH part
fit.g
#################################3
### the residuals of the garch() output are standardized. We square them to look
### for remaining conditional variance, we need to tell the acf() function to ignore
### the missing values from the residuals
acf(residuals(fit.g)^2, na.action=na.omit, lwd=2)
pacf(residuals(fit.g)^2, na.action=na.omit, lwd=2)

########################################
## Alternatively we can use the garchFit() function from fGarch
##  
## This function works a little different than the garch() function. For one, the order
## of the parameters are flipped. Second the residuals from this fit are NOT standardized
## but we can easily get the standardizing terms, the estimation h-t
fit.garch <- garchFit(~garch(1,1), data=fit$residuals, trace=FALSE)  ### garch(p,q)
####################################
## Note this acf plot is exactly the same as that from the fitted ARMA model
acf(fit.garch@residuals, lwd=2)
#######################3
## Standardize the residuals and square em.
acf(fit.garch@residuals^2/(fit.garch@h.t) , lwd=2)
fit.garch

###########
### Looks like we captured everything
### We can also statistically test this using a result from Li & Mak (1994)
### The distribution depends on the residuals and the standardizing terms
Weighted.LM.test(fit.garch@residuals, fit.garch@h.t, lag=15, fitdf=2)




##########################################################
##########################################################
## One cool thing with the garchFit() function is we can
## simultaneously fit the entire ARIMA+GARCH process in one
## big function. Keep in mind our ARIMA(0,0,2)(0,0,1)[12]
## is equivalent to an ARMA(0,14)
##
## There is no way to specify the seasonal ARMA model (to my knowledge)
## so we fit this massive ARMA.  It makes our goodness-of-fit
## test goofy because we lose sooo many degrees of freedom
big.fit <- garchFit(~arma(0,14)+garch(1,1), data=inflation, trace=FALSE)
summary(big.fit)
Weighted.Box.test(big.fit@residuals, lag=40, , fitdf=14, type="Ljung", weighted=FALSE)
Weighted.Box.test(big.fit@residuals, lag=30, type="Ljung", sqrd.res=TRUE)
Weighted.LM.test(big.fit@residuals, big.fit@h.t, lag=15, fitdf=2)



######################################################################
######################################################################
######################################################################
### Below is the Code where I compare the different models
###
### fit0 is the most basic statistical model, X = mu + error
### We estimate mu with x.bar and the error is used for error bands
###
### fit1, we fit X = mu + ARMA
### the ARMA process provides a slightly different model
### but has no real effect on the error bands
###
### fit2, we fit X = mu + ARMA + GARCH
### The mean prediction does not change from fit2 but our prediction
### intervals now account for the conditional changes in the variance



step.ahead<-48   ## How much to predict/forecast
recent.inflation <- window(inflation, start=c(2007,1))  # recent inflation for plotting

#########################
### Compare our models
#########################
# Fit NOTHING
fit0 <- Arima(inflation, order=c(0,0,0))
pred0 <- predict(fit0, n.ahead=step.ahead)
#### I will calculate the prediction errors myself
up0 <- pred0$pred + 1.96*pred0$se
down0 <- pred0$pred - 1.96*pred0$se
ts.plot(recent.inflation, pred0$pred, up0, down0, ylim=c(-1.75,1.25),
        col=c("black", "chocolate4", "chocolate4", "chocolate4"), lwd=c(1,2,2,2), lty=c(1,1,2,2))

#########################
### our seasonal MA model
fit1 <- Arima(inflation, order=c(0,0,2), seasonal=list(order=c(0,0,1),period=12))
pred1 <- predict(fit1, n.ahead=step.ahead)
up1 <- pred1$pred + 1.96*pred1$se
down1 <- pred1$pred - 1.96*pred1$se
ts.plot(recent.inflation, pred1$pred, up1, down1, ylim=c(-1.75,1.25),
        col=c("black", "blue", "blue", "blue"), lwd=c(1,2,2,2), lty=c(1,1,2,2))

#########################
## big MA(14) plus GARCH(1,1)
pred2 <- predict(big.fit, n.ahead=step.ahead)
up2 <- pred1$pred + 1.96*pred2$meanError
down2 <- pred1$pred - 1.96*pred2$meanError
up2 <- ts(up2, start=c(2015,4), freq=12)
down2 <- ts(down2, start=c(2015,4), freq=12)
ts.plot(recent.inflation, pred1$pred, up2, down2, ylim=c(-1.75,1.25),
        col=c("black", "darkgreen", "darkgreen", "darkgreen"), lwd=c(1,2,2,2), lty=c(1,1,2,2))

#############
## Now all on one plot

ts.plot(recent.inflation, pred0$pred, up0, down0,
        pred1$pred, up1, down1,pred1$pred, up2, down2, ylim=c(-1.75,1.25),
        col=c("black", rep("chocolate4",3), rep("blue",3), rep("darkgreen",3)),
        lwd=c(1,2,2,2,2,2,2,2,2,2),
        lty=c(1,1,2,2,1,2,2,1,2,2),
        main="Inflation Forecast", ylab="Inflation")
legend("bottomright", leg=c("Inflation Series", "IID Model", "ARMA", "ARMA+GARCH"),
       col=c("black", "chocolate4", "blue", "darkgreen"), lty=1, lwd=c(1,2,2,2) )

