require(tseries)
library(forecast)
require(vars)
############
## Load our Series
tmp <- read.csv("gdpUnemp.csv")

#############################
## Construct our Time Series objects
gdp <- ts(log(tmp[,2]), start=c(1980,1), frequency=4)
unemp <- ts(tmp[,3], start=c(1980,1), frequency=4)
### This creates a bivariate time series object
finance <- ts.union(gdp, unemp)
plot(finance, type="o", pch=20, col="gray30", main="Financial Series")

#########
## The basic idea from a long time ago
## decompose each series into the seasonal, trend and random part
## look at the cross-correlation between the random parts
## 
## Some indication GDP influences Unemployment...
gdp.dec <- stl(gdp, s.window="periodic")
unemp.dec <- stl(unemp, s.window="periodic")
ccf(gdp.dec$time.series[,3], unemp.dec$time.series[,3],
    main="CCF GDP & Unemployment", lwd=2)

################
## Let's start modeling our series marginally
##
## First we note both are not-stationary
## We look at first differences (to handle the stochastic trends)
## and lag-4 differences to handle the seasonality.
adf.test(gdp)
adf.test(unemp)
diff.finance <- diff(diff(finance, lag=4))
diff.finance <- diff(finance, lag=4)
plot(diff.finance, type="o", pch=20, col="gray30",
     main="Seasonally and Trend Adjusted Series (by differencing)")

############
## The data looks fairly stationary now
##
## Check for serial correlation
acf(diff.finance, lwd=2)
## Look at the cross-correlation more-closely
ccf(diff.finance[,1], diff.finance[,2],
    main="CCF GDP & Unemployment", ylab="CCF")

#############
## We see some serial correlation in the marignal series along with 
## interaction in the cross-correlation, recall from early April
## if we want to look at the CCF we should prewhiten each of our
## marginal series
acf(diff.finance[,1], lwd=2)
pacf(diff.finance[,1], lwd=2)
acf(diff.finance[,2], lwd=2)
pacf(diff.finance[,2], lwd=2)  ### Both resemble AR-processes
####
## Fit a larger AR(p) model to this data.
## Notes: this model is not necessarily optimal! I am just fitting something
## that captures most/all of the marginal serial correlations
gdp.model <- Arima(diff.finance[,1], order=c(5,0,0), include.mean=FALSE)
unemp.model <- Arima(diff.finance[,2], order=c(5,0,0), include.mean=FALSE)
acf(ts.union(gdp.model$residuals, unemp.model$residuals), lwd=2) ## Got most acf?
ccf(gdp.model$residuals, unemp.model$residuals, lwd=2,
    main="GDP & Unemployment (Prewhitened)", ylab="CCF")

#############
# let's try some Vector Autoregressive Models
VARselect(diff.finance)

##############
## Not entirely clear, some say AR(10). This seems pretty big
## given the data is quarterly? But one could argue it 
## also needs that far back because they are both measures of the
## economy which changes slowly?
my.var <- VAR(diff.finance,p=10)
my.var
summary(my.var)

###########################
## seems like overkill to me... Many non-significant terms
## 
## Let's try with p=4.  This is close to the 
## to the BIC (SC) criterion and the data is quarterly
## so it might capture some seasonal effects
my.var2 <- VAR(diff.finance, p=4)
my.var2
summary(my.var2)

### Which model is better? A matter of opinion I suppose
## 
## But with both models we can see something interesting
## Using the Granger test for causality
#######################
## Using the VAR(p=10) model
causality(my.var, cause="unemp")
causality(my.var, cause="gdp")

## Using the VAR(p=4) model
causality(my.var2, cause="unemp")
causality(my.var2, cause="gdp")

##############
## So we see the two series are instantaneously connected,
## This makes sense. Major spikes to the economy will have impact
## on both unemployment and GDP
##
## However we also see that GDP appears to "cause" unemployment.
## Furthermore, most of this causal effect is off by one-lage
## if you recall from the CCF plot.
##
## So, a decrease in GDP this quarter, will cause unemployment to 
## next quarter (also unemployment should decrease this quarter as well)
##
## Likewise, increase in GDP should cause unemployment to decrease
## In some of the economics literature this is known a Okun's Law

#######
## The vars packages has a neat feature
## in that it will plot your fitted model with the residuals and ACF/PACF
plot(my.var2)

###############
## Of course, we should look at the ACF of our 
## residuals to make sure we captured everything
## 
## For the interested reader, there are multivariate
## analogoues of the Ljung Box goodness-of-fit test, 
## see Hosking (1980)

## First the VAR(10)
acf(residuals(my.var))  # Nearly all serial correlation
acf(residuals(my.var2)) # not everything, but we got most of it

###############
## and we can use the fitted model to forecast
my.var.predict <- forecast(my.var, n.ahead=20, ci=0.95)
my.var2.predict <- forecast(my.var2, n.ahead=20, ci=0.95)
plot(my.var.predict, lwd=3)
plot(my.var2.predict, lwd=3)
