######
## Load the data and plot it
## But first load the external library, WeightedPortTest. 
data(lynx)
help(lynx)
plot(lynx, main="Annual Lynx Trappings", ylab="Captures")
log.lynx <- log(lynx)
plot(log.lynx, main="Annual Lynx Trappings", ylab="Log Captures")
acf(log.lynx)
pacf(log.lynx, lwd=3)

####
## The ACF is oscillating and decaying
## the PACF may cutoff but also there is indication it is decaying
##   PACF values are lag 7 and 11 appear significant.

###########
## Fit a large order AR model and check the validity of the fit
lynx.ar <- ar(log.lynx, method="mle", aic=TRUE)  ## Will use AIC to select the order p
lynx.ar
print(lynx.ar$order)
print(lynx.ar$ar)
print(lynx.ar$var.pred)
print(lynx.ar$x.mean)
####################
## My Full Model is
## X[t]-6.6678 = 1.167(X[t-1]-6.6678) - 0.5446(X[t-2]-6.6678) + 0.266(X[t-3]-6.6678) 
##              -0.309(X[t-4]-6.6678) + 0.154(X[t-5]-6.6678) - 0.1463(X[t-6]-6.6678)
##              +0.0569(X[t-7]-6.6678) - 0.029(X[t-8]-6.6678) + 0.1346(X[t-9]-6.6678)
##              +0.202(X[t-10]-6.6678) - 0.3394(X[t-11]-6.6678) + e[t]
##    where e[t] ~ WN(0, sigma^2 = 0.1914677)

# If we captured the serial correlation with the AR model, 
# the residuals should look like White Noise
plot(lynx.ar$resid, main="Lynx AR-fit Residuals", sub="White Noise?? - Cannot tell")
acf(lynx.ar$resid, na.action=na.omit)
pacf(lynx.ar$resid, na.action=na.omit)
######################
## The ACF & PACF plots tell us there is no serial correlation in the residuals
## so they are likely white noise.
##
## Therefore, using te AR(11) model, we captured the serial correlation in the 
## lynx data.

### You should notice that the fitted model is an AR(11)
### So the first 11 observations are needed to predict/model the 12th value
### 
lynx.fitted <- log.lynx - lynx.ar$resid
ts.plot(log.lynx, lynx.fitted, lty=1:2, lwd=1:2, col=c("black","red"), 
        main="Annual Lynx Trappings", ylab="Log Captures")


### Predicting using the forecast library
lynx.predict <- forecast(lynx.ar, h=20)
plot(lynx.predict)

plot(forecast(lynx.ar, h=100))

##############
## note as the forecast goes into the future the strong oscillation
## is decreasing.  Recall Lab #05... Ultimately, a forecast will
## eventually fall back to the mean.

lynx.mean.only <- Arima(log.lynx, order=c(0,0,0))
par(mfrow=c(2,1))
plot(forecast(lynx.ar, h=100))
abline(h=lynx.ar$x.mean)
plot(forecast(lynx.mean.only, h=100))
par(mfrow=c(1,1))

###########
## NOTE: In the AR model, the prediction interval
## is much smaller for the few few years than the mean-only model
## This mean-only model essentially is assuming IID data
## 
## The AR model uses the serial correlation to improve the forecast

###################
## Is an AR(11) too big?
##
## We discussed in class that a smaller order ARMA model may be preferred
## to an AR model because it might be simpler.
## Consider the following:

fit110 <- Arima(log.lynx, order=c(11,0,0))
fit11 <- Arima(log.lynx, order=c(1,0,1))
fit21 <- Arima(log.lynx, order=c(2,0,1))
fit12 <- Arima(log.lynx, order=c(1,0,2))
fit22 <- Arima(log.lynx, order=c(2,0,2))
fit32 <- Arima(log.lynx, order=c(3,0,2))
fit23 <- Arima(log.lynx, order=c(2,0,3))
fit33 <- Arima(log.lynx, order=c(3,0,3))

fit110  
fit11
fit21
fit12
fit22
fit32
fit23
fit33

###########################
## Looking at all the models, the AIC, AICc and BIC values
## We have several candidate models.  
## The AR(11) and the ARMA(3,3) seem like good candidates
##
## But we note in both models, some of the terms are insignificant
##  why???? Using MLE to get the model!
##  MLEs tend to be normal random variables
##  point.estimate +/- 1.96*SE
##  or approximately, 
##  point.estimate +/- 2*SE    
##  
## We can do a quick hypothesis test for signficance.
## 
## We can fix certain terms to be zero using the 'fixed' parameter in Arima()
## It must be of the same length as the number of terms fit, so in the first
## case that is 11+1  (11 AR terms plus the mean)
fit110.fix <- Arima(log.lynx, order=c(11,0,0), fixed=c(NA,NA,0,NA,0,0,0,0,0,0,NA,NA))
fit33.fix <- Arima(log.lynx, order=c(3,0,3), fixed=c(NA,NA,NA,NA,0,NA,NA))

### Let's compare
fit110
fit110.fix
fit33
fit33.fix
###############
## Do the models capture the serial correlation?
###############
## The original AR(11)
stats::acf(fit110$residuals)
stats::pacf(fit110$residuals)

## The AR(11) with insignificant terms removed
stats::acf(fit110.fix$residuals)
stats::pacf(fit110.fix$residuals)

## The ARMA(3,3) with insignificant term removed
stats::acf(fit33.fix$residuals)
stats::pacf(fit33.fix$residuals)

###################################
## How do the foreacst look?
par(mfrow=c(3,1))
plot(forecast(lynx.ar, h=20))
plot(forecast(fit110.fix, h=20))
plot(forecast(fit33.fix, h=20))
par(mfrow=c(1,1))

##########################
## All forecast are quite similar
## I would recommend using the ARMA(3,3) with theta_2 = 0
## Has smallest AIC, AICc, BIC.
## Only has 6 estimated parameters compared to the original AR(11) with 12.
## Yet appears to capture all/most of the serial correlation.



