library(tseries)            ## Had the Dickey-Fuller test in it
library(forecast)           ## We use the Arima() function from here
library(WeightedPortTest)   ## Has the goodness-of-fit test

################################
## Read in the data & Plot it
##
tmp <- read.csv("sheep.csv")
sheep <- ts(tmp[,2], start=1867)
plot(sheep, main="Annual Sheep Population", ylab="thousands", sub="in England and Wales", lwd=2, type="o", pch=20)

##########################
## Looks to be not stationary. Maybe a decreasing stochastic trend?

###########################
## Check for a unit root.
adf.test(sheep)
pp.test(sheep)
## Confirmed!
############################
## So the series looks non-stationary
## and the available Hypothesis test supports this claims.


#################################
## We transform the data as it may have an increasing variance with the mean
## We also take differences to get rid of the unit root.
log.sheep <- log10(sheep)
sheep.diff <- diff(log.sheep)

plot(sheep.diff, main="Changes in Annual Sheep Population", 
     type="o", lwd=2, pch=20, ylab=expression(paste(log[10], "(thousand sheep)")))
##
## From a visual perspective, this series seems stationary. 
## 
########################
adf.test(sheep.diff)
pp.test(sheep.diff)
########################
## the unit root tests confirm there is no unit root
## so it likely is stationary

#####################################################
## Explore the correlogram to determine if
## any serial correlation exist
acf(sheep.diff, lwd=2)
pacf(sheep.diff, lwd=2)

#############################################
## The ACF appears to be decaying
## the PACF may or may not cutoff at lag 3
## So perhaps an AR(3) or some ARMA(p,q) model
## will capture the serial correlation.

######################################
## Here I fit all possible combinations
## of ARMA(p,q) for up to lag 3

fit10 <- Arima(sheep.diff, order=c(1,0,0))
fit20 <- Arima(sheep.diff, order=c(2,0,0))
fit30 <- Arima(sheep.diff, order=c(3,0,0))
fit01 <- Arima(sheep.diff, order=c(0,0,1))
fit02 <- Arima(sheep.diff, order=c(0,0,2))
fit03 <- Arima(sheep.diff, order=c(0,0,3))
fit11 <- Arima(sheep.diff, order=c(1,0,1))
fit12 <- Arima(sheep.diff, order=c(1,0,2))
fit21 <- Arima(sheep.diff, order=c(2,0,1))
fit22 <- Arima(sheep.diff, order=c(2,0,2))
fit13 <- Arima(sheep.diff, order=c(1,0,3))
fit23 <- Arima(sheep.diff, order=c(2,0,3))
fit31 <- Arima(sheep.diff, order=c(3,0,1))
fit32 <- Arima(sheep.diff, order=c(3,0,2))
fit33 <- Arima(sheep.diff, order=c(3,0,3))

############################################
## This little function extracts the
## AIC, AICc and BIC values from an Arima() fit
getAIC <- function(fit) {
  c(fit$aic, fit$aicc, fit$bic)
}

############################
## Create a table with all the AIC, AICc & BIC values
tab <- rbind(getAIC(fit10), getAIC(fit20), getAIC(fit30),
             getAIC(fit01), getAIC(fit02), getAIC(fit03),
             getAIC(fit11), getAIC(fit12), getAIC(fit13),
             getAIC(fit21), getAIC(fit22), getAIC(fit23),
             getAIC(fit31), getAIC(fit32), getAIC(fit33) )
colnames(tab) <- c("AIC", "AICc", "BIC")
rownames(tab) <- c("AR(1)", "AR(2)", "AR(3)",
                   "MA(1)", "MA(2)", "MA(3)",
                   "ARMA(1,1)", "ARMA(1,2)", "ARMA(1,3)",
                   "ARMA(2,1)", "ARMA(2,2)", "ARMA(2,3)",
                   "ARMA(3,1)", "ARMA(3,2)", "ARMA(3,3)")
###########
## display our table!
tab

#######################
## from the table, I deem the following
## candidate models for the serial correlation
fit30
fit13
fit21
fit22
fit31
################################
## Overall I deem the AR(3) the best
## but it could be reduced
fit30.reduced <- Arima(sheep.diff, order=c(3,0,0), fixed=c(NA,0,NA,NA))
fit30.reduced

#################################
## It looks like we should not include
## a mean/intercept term either
fit30.noMean <- Arima(sheep.diff, order=c(3,0,0), fixed=c(NA,0,NA), include.mean=FALSE)
fit30.noMean

#################################
## Does the AR(3) capture all the serial correlation?
acf(fit30.noMean$residuals)
pacf(fit30.noMean$residuals)
Weighted.Box.test(fit30.noMean$residuals, type="Ljung", lag=20, fitdf=2)
Weighted.Box.test(fit30.noMean$residuals, type="Monti", lag=20, fitdf=2)

###############################
## So it appears we have a model. 
##
## Let's use Arima() to fit the log.sheep data
### the 1 in the (3,1,0) part tells the function
## to take first differences of the series
## then fit an AR(3) with the second term fixed to 0
Arima(log.sheep, order=c(3,1,0), fixed=c(NA,0,NA))

